package BATCH2UR::CondorParser;

use strict;
use warnings;
use BATCH2UR::ConversionFunctions;
use BATCH2UR::UsageRecordList;
use Data::Dumper;

#-----------------------------------------------------------------------
# Constructor (not yet used!!!)
#-----------------------------------------------------------------------
sub new {
    my $class = shift;
    my $self = {};
    bless($self, $class);
    return $self;
}

#-----------------------------------------------------------------------
# Parse Condor Log file
#-----------------------------------------------------------------------
sub parse {
    my $file = shift;
    my $urlist = BATCH2UR::UsageRecordList->new();

    open(Condor_LOG, "<$file") or die ("Unable to open $file error: $!");

    my $getJob;
    my $MyType;
    my $jobEntry=0;
    my %job;

    while ( my $line = <Condor_LOG>) {
        chomp $line;

        #Start Marker of Job
        if ( $line =~ m/^MyType = "(.*)"/ ) {
            $MyType=$1;
            if ( $MyType eq "Job") {
                $getJob=1;
            }else{
                $getJob=0;
            };
        };

        #Build a hash for each job
        if ( $getJob ) {
            if ( $line =~ m/(.*?) = (.*)/ ) {
                my $key=$1;
                my $value=$2;
                #Filter quotes if necessary
                $value =~ s/"(.*)"/$1/;
                $job{$key} = $value;
            };
        };

        #End Marker of job
        if ( $line =~ m/^\*\*\* .*/ ) {
            if ( $getJob ) {
                # Add to Usage Record
                my $ur = $urlist->addRecord($job{ClusterId});

                $ur->LocalUserId($job{Owner});
                $ur->JobName($job{ClusterId});

                if ( $job{JobStatus} == 4 ) {
                    $ur->Status("completed");
                }else {
                    $ur->Status("failed");
                }

                $ur->timeCondorSubmitted(epochToDate($job{QDate}));
                $ur->WallDuration(secondsToDuration($job{RemoteWallClockTime}));
                $ur->CpuDuration(secondsToDuration($job{RemoteSysCpu}+$job{RemoteUserCpu}));

                #This function needs to edited on a per-site per-parallel universe
                #basis. For example
                #
                #if ($job{CurrentHosts} > 1) {
                #    my $procs = $job{CurrentHosts} * 4;
                #}
                #$ur->Processors($procs);
                #
                #However as a default we assume Condor only allows serial jobs...
                $ur->Processors(1);
                $ur->StartTime(epochToDate($job{JobStartDate}));
                $ur->EndTime(epochToDate($job{CompletionDate}));
            };

                # Clear markers and job hash
                $jobEntry++;
                $getJob=0;
                %job=();
                #if ($jobEntry == 1 ) {
                #    exit;
                #};
            }
            $MyType=0;
        };

    return $urlist;

};

1;

__END__

=head1 NAME

CondorParser - Perl extension for parsing the execution records of a
Condor accounting log.

=head1 SYNOPSIS

  use CondorParser;

=head1 ABSTRACT

  CondorParser returns a hash of hashs, keyed by Condor JobID. The data
  hash contains the variables extracted from Execution records from
  the passed in file. These are used by BATCH2UR::Generate to populate the 
  TT template.

=head1 DESCRIPTION

parse() takes one argument the Condor accounting log file.

=head2 EXPORT

None by default.

=head1 SEE ALSO


=head1 AUTHOR

Matt Ford, E<lt>matt.ford@manchester.ac.ukE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2010 by Matt Ford

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

