package BATCH2UR::Config;

use 5.008;
use strict;
use warnings;
use AppConfig qw/:argcount/;
#use Data::Dumper;

require Exporter;

our @ISA = qw(Exporter);
our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw($config);

our $config = AppConfig->new();

$config->define(
                'SubmitHost' => { 
                                 ARGCOUNT => ARGCOUNT_ONE,
                                 DEFAULT => "",
                                 VALIDATE => '\w+'
                                },
                'MachineName' => { 
                                  ARGCOUNT => ARGCOUNT_ONE,
                                  DEFAULT => "",
                                  VALIDATE => '\w+'
                                 },
                'SiteName' => { 
                                  ARGCOUNT => ARGCOUNT_ONE,
                                  DEFAULT => "",
                                  VALIDATE => '\w+'
                                 },
                'BatchSystem' => { 
                                  ARGCOUNT => ARGCOUNT_ONE,
                                  DEFAULT => "PBS",
                                  VALIDATE => '\w+'
                                 },
                'GlobusAccountingDir' => { 
                                          ARGCOUNT => ARGCOUNT_ONE,
                                          DEFAULT => "",
                                          VALIDATE => sub { return 1 if -d $_[1] }
                                          },
                'LcasAccountingDir' => { 
                                          ARGCOUNT => ARGCOUNT_ONE,
                                          DEFAULT => "",
                                          VALIDATE => sub { return 1 if -d $_[1] }
                                          },
                'LcasDatabaseDir' => { 
                                         ARGCOUNT => ARGCOUNT_ONE,
                                         DEFAULT => "",
                                         VALIDATE => sub { return 1 if -d $_[1] }
                                        },
                'LcasDatabaseCache' => { 
                                      ARGCOUNT => ARGCOUNT_ONE,
                                      DEFAULT => "67108864",
                                      VALIDATE => '^\d+$'
                                     },
                'BatchDatabaseDir' => { 
                                         ARGCOUNT => ARGCOUNT_ONE,
                                         DEFAULT => "",
                                         VALIDATE => sub { return 1 if -d $_[1] }
                                        },
                'BatchDatabaseCache' => { 
                                         ARGCOUNT => ARGCOUNT_ONE,
                                         DEFAULT => "67108864",
                                         VALIDATE => '^\d+$'
                                        },
                'BatchIndexCache' => { 
                                      ARGCOUNT => ARGCOUNT_ONE,
                                      DEFAULT => "33444432",
                                      VALIDATE => '^\d+$'
                                     },
                'GridMapDir' => { 
                                 ARGCOUNT => ARGCOUNT_ONE,
                                 DEFAULT => "",
                                 VALIDATE => sub { return 1 if -d $_[1] }
                                },
                'GridMapFile' => { 
                                  ARGCOUNT => ARGCOUNT_ONE,
                                  DEFAULT => "",
                                  VALIDATE => sub { return 1 if -f $_[1] }
                                 },
                'MakeGridMapFile' => { 
                                  ARGCOUNT => ARGCOUNT_ONE,
                                  DEFAULT => "",
                                  VALIDATE => sub { return 1 if -f $_[1] }
                                 },
                'BatchAccountingDir' => { 
                                         ARGCOUNT => ARGCOUNT_ONE,
                                         DEFAULT => "",
                                         VALIDATE => sub { return 1 if -d $_[1] }
                                        },
                'UrFileDir' => { 
                                ARGCOUNT => ARGCOUNT_ONE,
                                DEFAULT => "/var/spool/batch2ur",
                                VALIDATE => sub { return 1 if ( 
                                                               -d "$_[1]"  and
                                                               -d "$_[1]/upload" ) }
                               },
                'UrTemplateDir' => { 
                                ARGCOUNT => ARGCOUNT_ONE,
                                DEFAULT => "/usr/share/batch2ur",
                                VALIDATE => sub { return 1 if ( 
                                                               -d "$_[1]"  and
                                                               -d "$_[1]/templates" ) }
                               },
								
				'GatekeeperLogFile' => {
									ARGCOUNT => ARGCOUNT_ONE,
                                    DEFAULT => "/var/log/globus-gatekeeper.log",
                                    VALIDATE => sub { return 1 if -f $_[1] }
                                   },
				'LcmapsJobRepHostName' => {
                                    ARGCOUNT => ARGCOUNT_ONE,
                                    DEFAULT => "localhost",
                                    VALIDATE => '\w+'
                                   },
                'LcmapsJobRepDatabaseName' => {
                                     ARGCOUNT => ARGCOUNT_ONE,
                                     DEFAULT => "",
                                     VALIDATE => '\w+'
                                    },
                 'LcmapsJobRepDatabaseUsername' => {
                                     ARGCOUNT => ARGCOUNT_ONE,
                                     DEFAULT => "",
                                     VALIDATE => '\w+'
                                    },
 
                'LcmapsJobRepDatabasePassword' => {
                                     ARGCOUNT => ARGCOUNT_ONE,
                                     DEFAULT => "",
                                     VALIDATE => '\w+'
                                    },
 
               );
die "Error with config file or referenced files/dirs don't exist\n" unless $config->file($ENV{BATCH2UR});

#sub dumpConfig {
#  my %varlist = $config->varlist('.*');
#  foreach my $varname (keys %varlist) {
#    print "Variable name $varname, value = ", 
#         Dumper($config->get($varname)), "\n";
#  }
#};

1;
__END__
                # Below is stub documentation for your module. You'd better edit it!

=head1 NAME

BATCH2UR:Config - Read configuration file for use by the client

=head1 AUTHOR

Matt Ford, E<lt>matt.ford@manchester.ac.ukE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2006 by Matt Ford

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut
