package BATCH2UR::ConversionFunctions;

use 5.008;
use strict;
use warnings;
use Socket;
use Time::Local;

require Exporter;

our @ISA = qw(Exporter);
our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw(epochToDate timeToDuration pbsToDate pbsLocalToDate globusToDate 
                 globusAuditToDate lsfLocalToDate ipToHostName secondsToDuration);

sub epochToDate {
  my $epoch=shift;
  (my $sec,my $min,my $hour,my $mday,my $mon,my $year,my $wday,my $yday)=gmtime($epoch);
  $year+=1900;
  $mon+=1;
  $mon = sprintf("%02d", $mon);
  $mday = sprintf("%02d", $mday);
  $hour = sprintf("%02d", $hour);
  $min = sprintf("%02d", $min);
  $sec = sprintf("%02d", $sec);
  return $year."-".$mon."-".$mday."T".$hour.":".$min.":".$sec."Z";
};

sub timeToDuration {
  my $time=shift;
  (my $hours, my $minutes, my $secs) = split /:/,$time,3;
  $hours = sprintf("%02d", $hours);
  $minutes = sprintf("%02d", $minutes);
  $secs= sprintf("%02d", $secs);
  return "PT".$hours."H".$minutes."M"."$secs"."S";
};

sub secondsToDuration {
  my $seconds=shift;
  my $hours = sprintf("%02d", $seconds / 3600);
  $seconds = $seconds % 3600;
  my $minutes = sprintf("%02d", $seconds / 60);
  $seconds = $seconds % 60;
  $seconds = sprintf("%02d", $seconds);
  return "PT".$hours."H".$minutes."M"."$seconds"."S";
};

sub pbsToDate {
  (my $date, my $time)=split / /,shift,2;
  (my $month, my $day, my $year)= split /\//,$date,3;
  return "$year-$month-$day"."T".$time."Z";
};

sub pbsLocalToDate {
    (my $date, my $time)=split / /,shift,2;
    (my $month, my $day, my $year)= split /\//,$date,3;
    (my $hour, my $min, my $sec) = split /:/,$time,3;
    $month-=1;
    # Convert from our local time schema to time from epoch
    my $epoch=timelocal($sec,$min,$hour,$day,$month,$year);
    # Convert back but use gmtime instead
    return epochToDate($epoch);
};
sub globusToDate {
  (my $date, my $time)=split / /,shift,2;
  (my $year, my $month, my $day)= split /\//,$date,3;
  (my $hour, my $min, my $sec) = split /:/,$time,3;
  # Convert from our local time schema to time from epoch
  $month-=1;
  my $epoch=timelocal($sec,$min,$hour,$day,$month,$year);
  # Convert back but use gmtime instead
  return epochToDate($epoch);
};
sub globusAuditToDate {
  (my $date, my $time)=split / /,shift,2;
  (my $year, my $month, my $day)= split /-/,$date,3;
  return "$year-$month-$day"."T".$time."Z";
};

sub ipToHostName {
    my $ip = shift;
    my $ipaddr = inet_aton($ip);
    my $claimed_host = gethostbyaddr($ipaddr, AF_INET);
    if ( $claimed_host ) { 
	return $claimed_host;
    }else{
	return "";
    };
};

1;
__END__
=head1 NAME

ConversionFunctions - Perl extension for converting dates and times from and
to values used in the UR XML schema.

=head1 SYNOPSIS

  use ConversionFunctions;

=head1 ABSTRACT

This module converts to and from the formats of data found in the UR schema
and those found in Batch Systems, Globus and UNIX.

=head1 DESCRIPTION

This module converts to and from the formats of data found in the UR schema
and those found in PBS, Globus and UNIX.

=head2 EXPORT

epochToDate 
timeToDuration 
pbsToDate 
lsfToDate
globusToDate

=head1 SEE ALSO


=head1 AUTHOR

Matt Ford, E<lt>matt.ford@manchester.ac.ukE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2006 by Matt Ford

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut
