package BATCH2UR::GlobusParser;

use 5.008;
use strict;
use warnings;
use BerkeleyDB;
use MLDBM qw(BerkeleyDB::Hash Storable);
use BATCH2UR::Config;
use BATCH2UR::ConversionFunctions;

require Exporter;

our @ISA = qw(Exporter);
our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw(GlobusParser);

my $db_cache=$config->BatchDatabaseCache; # Default is 64MB
my $idx_cache=$config->BatchIndexCache; # Default is 32MB
my $databaseDir=$config->BatchDatabaseDir;
my $batchSystem=lc($config->BatchSystem); # Always use lowercase

if ( $batchSystem eq "torque" ) {
  $batchSystem = "pbs"; # globus log accounting makes no distinction between pbs and torque
};

my $database="$databaseDir/$batchSystem.db";
my $index="$databaseDir/$batchSystem.idx";


my %database=();
my %index=();

if (-d $databaseDir ) {
  if ($database) {
    tie %database, "MLDBM", -Filename => $database, -Flags => DB_INIT_LOCK , 
      -Cachesize => $db_cache
	or die "Cannot open file $database: $! $BerkeleyDB::Error\n";
  }; 
  if ($index) {
    tie %index, "MLDBM", -Filename => $index, -Flags => DB_INIT_LOCK , 
      -Cachesize => $idx_cache 
	or die "Cannot open file $index: $! $BerkeleyDB::Error\n";
  };
};

sub GlobusParser {
  return () unless( $db_cache and $idx_cache and $databaseDir);
  my $jobid=shift;
  if ( ! $index{$jobid} ) { return (); }
  if ( ! $database{$index{$jobid}} ) { return (); }
  my %jobvars=%{$database{$index{$jobid}}};
  my %globus=(
    "timeGlobusSubmitted"=>globusToDate($jobvars{"SUBMITTIME"}),
    "timeGlobusExecutable"=>globusToDate($jobvars{"EXECTIME"}),
    "timeGlobusFinished"=>globusToDate($jobvars{"FINISHTIME"}),
    "dn"=>$jobvars{"DN"},
    "SubmitHost"=>ipToHostName($jobvars{"SUBMIT_HOST"}),
    "jobTypeId"=>$index{$jobid},
    );
  return %globus;
};
1;

__END__
# Below is stub documentation for your module. You'd better edit it!
=head1 NAME

BATCH2UR::GlobusParser - Reads Berkeley database of captured Globus logs.

=head2 EXPORT

GlobusParser(jobid) - get hash of Globus data by jobid.

=head1 AUTHOR

Matt Ford, E<lt>matt.ford@manchester.ac.ukE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2006 by Matt Ford

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut
