package BATCH2UR::GridmapParser;

use 5.008;
use strict;
use warnings;
use File::Basename;
use BATCH2UR::Config;

no warnings 'uninitialized';

require Exporter;

our @ISA = qw(Exporter);
our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw(GridmapParser);

my $gridmapdir=$config->GridMapDir;
my $gridmapfile=$config->GridMapFile;

my (%dn,%nd,%account,%tnuocca);
my (%gm_dn,%gm_account);

sub tidydn {
  my $string=shift;
  my $dn="";
  my @dn = split("%",$string);
  for my $i (@dn) {
    next if ($i eq "");
    my $str = chr(hex(substr($i,0,2)));
    $dn .= $str;
    $str = substr($i,2);
    $dn .= $str;
  }
  return $dn;
}

# Build lookup hashes (done for speed)
# Use gridmapd
if ($gridmapdir) {
  for (<$gridmapdir/*>) {
   my @inode=stat $_;
   my $inode=$inode[1];
   my $file=basename($_);
   if ($file=~m/^%2fc%/) {
     my $tdn=tidydn($file);
     $dn{$inode}=$tdn;
     $nd{$tdn}=$inode;
   }else{
     $account{$inode}=$file;
     $tnuocca{$file}=$inode;
     }
   }
}
# Use gridmapfile
if ($gridmapfile) {
    open (GRIDMAPFILE,"<$gridmapfile");
    while(<GRIDMAPFILE>) {
	chomp;
	# Skip comments
	if ( /^#.*/ ) {
	     next;
	 };
	my ($gm_dn,$gm_account) = /^"(.*)" (.*)/;
	# Skip pool accounts
	if ( $gm_account =~ /^\..*/ ) {
	    next;
	};
	# Take first account only if multiple accounts are assigned to a DN.
	($gm_account,my $remainder) = split(/,/,$gm_account);
	# If dn all ready mapped then keep the first entry
	if ($gm_dn{$gm_dn}) {
	    next;
	};
	$gm_dn{$gm_dn}=$gm_account;
	$gm_account{$gm_account}=$gm_dn;
    };
};

# account2dn
sub account2dn {
  my $account=shift;
  # Pool account lookups first
  if ($tnuocca{$account}) {
    if ($dn{$tnuocca{$account}}) {
      return $dn{$tnuocca{$account}};
    }
  }
  # Failover to grid-mapfile
  if ($gm_account{$account}) {
      return $gm_account{$account};
  }
  return 0;
}

# dn2account
sub dn2account {
  my $dn=shift;
  # Pool account lookups first
  $dn=~tr /[A-Z]/[a-z]/;
  if ($nd{$dn}) {
    if ($account{$nd{$dn}}) {
      return $account{$nd{$dn}};
    }
  }
  # Failover to grid-mapfile
  if ($gm_dn{$dn}) {
      return $gm_dn{$dn};
  }
  return 0;
}

sub GridmapParser {
  return unless ( -d $gridmapdir or -f $gridmapfile );
  my $dn=account2dn(shift);
  if ($dn) {
    return ("dn"=>$dn);
  }else{
    return ();
  };
};

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

BATCH2UR::GridmapParser

=head1 ABSTRACT

Builds up lookup hashes of user's to DN and vice versa

=head2 EXPORT

GridmapParser - takes a username and returns a DN in the form of hash ("dn"=>$dn)

=head1 AUTHOR

Matt Ford, E<lt>matt.ford@manchester.ac.ukE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2006 by Matt Ford

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut
