package BATCH2UR::LSFParser;

use strict;
use warnings;
use BATCH2UR::ConversionFunctions;
use BATCH2UR::UsageRecordList;

#-----------------------------------------------------------------------
# Constructor (not yet used!!!)
#-----------------------------------------------------------------------
sub new {
  my $class = shift;
  my $self = {};
  bless($self, $class);
  return $self;
}

#-----------------------------------------------------------------------
# Parse LSF Log file
# For details of format see man lsb.acct
#-----------------------------------------------------------------------
sub parse {
  my $file = shift;
  my $urlist = BATCH2UR::UsageRecordList->new();
  my %lsfFields=(
		 "0"=>"eventType",
		 "1"=>"version",
		 "2"=>"eventTime",
		 "3"=>"jobId",
		 "4"=>"userId",
		 "5"=>"options",
		 "6"=>"numProcessors",
		 "7"=>"submitTime",
		 "8"=>"beginTime",
		 "9"=>"termTime",
		 "10"=>"startTime",
		 "11"=>"userName",
		 "12"=>"queue",
		 "13"=>"resReq",
		 "14"=>"dependCond",
		 "15"=>"preExecCmd",
		 "16"=>"fromHost",
		 "17"=>"cwd",
		 "18"=>"inFile",
		 "19"=>"outFile",
		 "20"=>"errFile",
		 "21"=>"jobFile",
		 "22"=>"numAskedHosts",
		 "23"=>"askedHosts",
		 "24"=>"numExHosts",
		 "25"=>"execHosts",
		 "26"=>"jStatus",
		 "27"=>"hostFactor",
		 "28"=>"jobName",
		 "29"=>"command",
		 "30"=>"ru_utime",
		 "31"=>"ru_stime",
		 "32"=>"ru_maxrss",
		 "33"=>"ru_ixrss",
		 "34"=>"ru_ismrss",
		 "35"=>"ru_idrss",
		 "36"=>"ru_isrss",
		 "37"=>"ru_minflt",
		 "38"=>"ru_majflt",
		 "39"=>"ru_nswap",
		 "40"=>"ru_inbloc",
		 "41"=>"ru_oubloc",
		 "42"=>"ru_ioch",
		 "43"=>"ru_msgsnd",
		 "44"=>"ru_msgrcv",
		 "45"=>"ru_nsignals",
		 "46"=>"ru_nvcsw",
		 "47"=>"ru_nivcsw",
		 "48"=>"ru_exutime",
		 "49"=>"mailUser",
		 "50"=>"projectName",
		 "51"=>"exitStatus",
		 "52"=>"maxNumProcessors",
		 "53"=>"loginShell",
		 "54"=>"timeEvent",
		 "55"=>"idx",
		 "56"=>"maxRMem",
		 "57"=>"maxRSwap",
		 "58"=>"inFileSpool",
		 "59"=>"commandSpool",
		 "60"=>"rsvId",
		 "61"=>"sla",
		 "62"=>"exceptMask",
		 "63"=>"additionalInfo",
		 "64"=>"exitInfo",
		 "65"=>"warningAction",
		 "66"=>"warningTimePeriod",
		 "67"=>"chargedSAAP",
		 "68"=>"licenseProject",
		);
  my $e = qr/(-1|""|-1.0+)\s+/xms;	 # Match empty fields
  my $s = qr/"((?:[^"]+|"")*)"\s*/xms; # Match a string of the form "" or " "" "" "
  my $d = qr/(\d+)\s+/xms;	       # Match an int
  my $f = qr/(\d+\.*\d+)\s+/xms;       # Match a float

  open (LSF_LOG, "<$file") or die ("Unable to open $file error: $!");
  while ( my $input = <LSF_LOG>) {
    chomp $input;
    my %lsf;
    my $counter=0;
  LINE: while (length $input > 0 ){
      if ($input =~ s{\A $e}{}xms) {
	$lsf{$lsfFields{$counter}}=$1;
	$counter++;
      } elsif ($counter==23 or $counter==25) {
	#There is some crazyness with some fields depending on the
	# values of the preceeding ones
	# as to whether they are present or not.  This makes parsing a pain....
	#We deal for askedHosts and execHosts
	my $num=$lsf{$lsfFields{$counter-1}}; #How many fields to expect.
	if ($num==0) {
	  #No extra fields present so move on....
	  $lsf{$lsfFields{$counter}}="";
	} else {
	  #Now we have an additional $num fields skewing all the others......
	  for (1..$num) {
	    $input =~ s{\A $s}{}xms;
	    $lsf{$lsfFields{$counter}} .= "$1:";
	  }
	}
	$counter++;
      } elsif ($input =~ s{\A $s}{}xms) {
	$lsf{$lsfFields{$counter}}=$1;
	$counter++;
      } elsif ($input =~ s{\A $d}{}xms) {
	$lsf{$lsfFields{$counter}}=$1;
	$counter++;
      } elsif ($input =~ s{\A $f}{}xms) {
	$lsf{$lsfFields{$counter}}=$1;
	$counter++;
      } else {
	die "Error parsing accounting file at field $counter";
      }
      last LINE if  ($lsf{eventType} ne "JOB_FINISH");
    }
    ;
    #At this point we have a hash %lsf with the fields populated
    if ($lsf{eventType} eq "JOB_FINISH") {
      # Add $ur record to list and populate with parsed data
      my $ur = $urlist->addRecord($lsf{jobId});
      $ur->logDate(epochToDate($lsf{eventTime}));
      if (defined $lsf{userName}) {
	$ur->LocalUserId($lsf{userName});
      };
      if (defined $lsf{jobName}) {
	$ur->JobName($lsf{jobName});
      };
      if ( defined $lsf{jStatus} and $lsf{jStatus} eq "64" ) {
	$ur->Status("completed");
      } else {
	#No sensible time in the UR for below....
	#LSF has very limited status reporting
	$ur->Status("exited");
      }
      ;
      if (defined $lsf{maxRMem} and $lsf{maxRMem} ne "-1") {
	$ur->memoryUsed($lsf{maxRMem},"KB");
      };
      if (defined $lsf{maxRSwap} and $lsf{maxRSwap} ne "-1") {
	$ur->virtualMemoryUsed($lsf{memRSwap},"KB");
      };
      #Not sure if we have an equivalent value in LSF accounting file...
      #$ur->wallTimeRequested(timeToDuration($fieldValue));
      if (defined $lsf{submitTime} and $lsf{submitTime} ne "0") {
	  $ur->timeLsfSubmitted(epochToDate($lsf{submitTime}));
      };
      if (defined $lsf{startTime}){
	  if ($lsf{startTime} ne "0") {
	      $ur->timeLsfExecutable(epochToDate($lsf{startTime}));
	  }else{
	      $ur->StartTime("INVALID:". epochToDate($lsf{startTime}));
	  };
      };
      if (defined $lsf{eventTime} and defined $lsf{startTime}) {
	  if ($lsf{eventTime} ne "0" and $lsf{startTime} ne "0") {
	      $ur->WallDuration(secondsToDuration($lsf{eventTime}-$lsf{startTime}));
	  }else{
	      $ur->WallDuration(secondsToDuration(0));
	  };
      };
      if (defined $lsf{ru_time} and $lsf{ru_time} ne "-1"
	  and defined $lsf{ru_stime} and $lsf{ru_stime} ne "-1") {
	$ur->CpuDuration(secondsToDuration($lsf{ru_utime}+$lsf{ru_stime}));
      };
      if (defined $lsf{maxNumProcessors} and $lsf{maxNumProcessors} ne "-1") {
	$ur->Processors($lsf{maxNumProcessors});
      };
      if (defined $lsf{eventTime}) {
	  if ($lsf{eventTime} ne "0") {
	      $ur->EndTime(epochToDate($lsf{eventTime}));
	  }else{
	      $ur->EndTime("INVALID:". epochToDate($lsf{eventTime}));
	  };
      };
      if (defined $lsf{startTime}){
	  if ($lsf{startTime} ne "0") {
	      $ur->StartTime(epochToDate($lsf{startTime}));
	  }else{
	      $ur->StartTime("INVALID:". epochToDate($lsf{startTime}));
	  };
      };
      if (defined $lsf{queue}) {
	$ur->Queue($lsf{queue});
      };
      if (defined $lsf{execHosts}) {
	my @nodes= split /:/,$lsf{execHosts};
	$ur->NodeCount(scalar(@nodes));
	$ur->Host(@nodes);
      };
    }
    ;
  }
  ;
  return $urlist;
};

1;

__END__

=head1 NAME

LSFParser - Perl extension for parsing the execution records of a 
LSF accounting log.

=head1 SYNOPSIS

  use LSFParser;

=head1 ABSTRACT

  LSFParser returns a hash of hashs, keyed by LSF JobID. The data
  hash contains the variables extracted from Execution records from
  the passed in file. These are used by BATCH2UR::Generate to populate the 
  TT template.

=head1 DESCRIPTION

parse() takes one argument the LSF accounting log file.

=head2 EXPORT

None by default.

=head1 SEE ALSO


=head1 AUTHOR

Matt Ford, E<lt>matt.ford@manchester.ac.ukE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2006 by Matt Ford

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut
