package BATCH2UR::LcasParser;

use 5.008;
use strict;
use warnings;
use BerkeleyDB;
use MLDBM qw(BerkeleyDB::Hash Storable);
use BATCH2UR::Config;
use BATCH2UR::ConversionFunctions;

require Exporter;

our @ISA = qw(Exporter);
our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw(LcasParser);

my $db_cache=$config->LcasDatabaseCache; # Default is 64MB
my $databaseDir=$config->LcasDatabaseDir;
my $database="$databaseDir/lcas-jobmanager.db";

my %database=();

if (-d $databaseDir ) {
  if ($database) {
    tie %database, "MLDBM", -Filename => $database, -Flags => DB_INIT_LOCK , 
      -Cachesize => $db_cache
	or die "Cannot open file $database: $! $BerkeleyDB::Error\n";
  };
};

sub LcasParser {
  return () unless( $db_cache and $databaseDir);
  my $jobtypeid=shift;
  if ( ! $database{$jobtypeid} ) { return (); }
  my %lcasvars=%{$database{$jobtypeid}};
  my %lcas=(
            "vo0"=>$lcasvars{VO0},
           );
return %lcas;
};
1;

__END__
# Below is stub documentation for your module. You'd better edit it!
=head1 NAME

BATCH2UR::LcasParser - Reads Berkeley database of captured Lcas (VOMS) logs.

=head2 EXPORT

LcasParser(jobTypeId) - get hash of lcas data by jobtypeid.

=head1 AUTHOR

Matt Ford, E<lt>matt.ford@manchester.ac.ukE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2009 by Matt Ford

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut
