package BATCH2UR::LcmapsParser;

use 5.008;
use strict;
use warnings;
use DBI;
use BATCH2UR::Config;
use BATCH2UR::ConversionFunctions;

no warnings 'uninitialized';

require Exporter;

our @ISA = qw(Exporter);
our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw(LcmapsParser);

my $GatekeeperLogFile=$config->GatekeeperLogFile;
my $LcmapsJobRepHostName=$config->LcmapsJobRepHostName;
my $LcmapsJobRepDatabaseName=$config->LcmapsJobRepDatabaseName;
my $LcmapsJobRepDatabaseUsername=$config->LcmapsJobRepDatabaseUsername;
my $LcmapsJobRepDatabasePassword=$config->LcmapsJobRepDatabasePassword;

# Requires lcmaps-plugins-jobrep

sub LcmapsParser {
	#print "LcmapsParser - Job lookup\n";
	#print " (Local)JobId: $_[0] \n"; # e.g. 38.sif.nikhef.nl.xml
	
	#Scrape the Gatekeeper log to extract the GATEKEEPER_JM_ID based on the 
	#GRAM_SCRIPT_JOB_ID, which is the LocalJobId
	
	open my $fh_gatekeeperLog, $GatekeeperLogFile or die "Could not open Globus gatekeeper log: $GatekeeperLogFile"; 
	my @lines = grep /$_[0]/, <$fh_gatekeeperLog>;                                               
	my @parts = split(/ /, $lines[0]);                                                                     
	my $gatekeeper_jm_id = $parts[4];                                                                      
	my $dbh = DBI->connect("dbi:mysql:$LcmapsJobRepDatabaseName;host=$LcmapsJobRepHostName","$LcmapsJobRepDatabaseUsername","$LcmapsJobRepDatabasePassword") or die "Connection Error: $DBI::errstr\n";
	
	#Pull the effective VOMS attributes from the lcmaps-jobrep database
	my $sql = " 
	SELECT 
	voms_fqans.fqan
FROM 
	compute_jobs
	LEFT OUTER JOIN effective_credentials_unix_uid_voms ON compute_jobs.eff_cred_id = effective_credentials_unix_uid_voms.eff_cred_id
	LEFT OUTER JOIN unix_uid_voms_fqans ON effective_credentials_unix_uid_voms.unix_uid_voms_fqan_id = unix_uid_voms_fqans.unix_uid_voms_fqan_id
	LEFT OUTER JOIN voms_fqans ON voms_fqans.voms_fqan_id = unix_uid_voms_fqans.voms_fqan_id
WHERE
	compute_jobs.gatekeeper_jm_id = '$gatekeeper_jm_id';";
	
	my $sth = $dbh->prepare($sql);
	$sth->execute or die "SQL Error: $DBI::errstr\n";
	my $row = $sth->fetchrow_hashref;

	my @fqan_parts = split(/\//, $row->{fqan});
	# VO
	my $lcmaps_vo = $fqan_parts[1];
	
	# VO Group
	my $lcmaps_vo_group;
	if($fqan_parts[2] !~ m/^role.+/i)
	{
		$lcmaps_vo_group = $fqan_parts[2];
		shift @fqan_parts;
	}
	else 
	{	
		$lcmaps_vo_group = "";
	}; 
	shift @fqan_parts;
	
	# VO Role
	my $lcmaps_vo_role;
	foreach (@fqan_parts)
	{
		if($_ =~ m/^role/i)
		{
			my @vo_role_parts = split(/=/, $_);
			$lcmaps_vo_role = $vo_role_parts[1];
		}
	}
	
	my %lcmaps=( 
		"lcmaps_vo"=>$lcmaps_vo,
		"lcmaps_vo_group"=>$lcmaps_vo_group,
		"lcmaps_fqan"=>$row->{fqan},
		"lcmaps_vo_role"=>$lcmaps_vo_role,
 	);
	
	return %lcmaps;  
}

1;
__END__
# Below is stub documentation for your module. You'd better edit it!
=head1 NAME

BATCH2UR::LcmapsParser - Capture VOMS data from lcmaps-plugins-jobrep database.

=head2 EXPORT

LcmapsParser(jobTypeId) - get hash of lcmaps data by jobtypeid.

=head1 AUTHOR

John Robinson, E<lt>J.S.Robinson.@soton.ac.ukE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2013 by IGE Project

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut
