package BATCH2UR::MakeGridmapParser;

use 5.008;
use strict;
use warnings;
use File::Basename;
use BATCH2UR::Config;
use Data::Dumper;

require Exporter;

our @ISA = qw(Exporter);
our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw(MakeGridmapParser);

my $makegridmapfile=$config->MakeGridMapFile;
my @accounts;

# Use makegridmapfile to build an array of pairs mapping accounts to
# voinfo. Have to use an array as order matters.
if ($makegridmapfile) {
  open (MAKEGRIDMAPFILE,"<$makegridmapfile");
  while(my $line = <MAKEGRIDMAPFILE>) {
    chomp $line;
    if ( $line =~ m/^#.*/o ) { next; };    # Skip comments
    my ($voinfo, $groups) = $line =~ m/group +(.*?) +(.*)/o;
    if ( $groups ) {
      my @groups=split(/,/,$groups);
      foreach my $group (@groups) {
        push @accounts, [$group,$voinfo];
      };
    };
  };
};

sub MakeGridmapParser {
  return unless ( @accounts ) ;
  my $user=shift;
  # Search pool account prefixs for one that matches with start of user.
  foreach my $pair ( @accounts ) {
    my ($account, $voinfo) = @$pair;
    # Direct account mapping
    if ( $user eq $account ) {
      return ( vo0 => $voinfo );
    };
    # Pool account mapping
    if ( $account =~ s/^\.//o ) {
      if ( $user =~ m/$account/ ) {
        return ( vo0 => $voinfo );
      };
    };
  };
  return;
};

1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

BATCH2UR::MakeGridmapParser

=head1 ABSTRACT

Builds up list of pool account source servers.

=head2 EXPORT

MakeGridmapParser - takes a username and returns voinfo string

=head1 AUTHOR

Matt Ford, E<lt>matt.ford@manchester.ac.ukE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2009 by Matt Ford

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut
