package BATCH2UR::SGEParser;

use strict;
use warnings;
use BATCH2UR::ConversionFunctions;
use BATCH2UR::UsageRecordList;

#-----------------------------------------------------------------------
# Constructor (not yet used!!!)
#-----------------------------------------------------------------------
sub new {
    my $class = shift;
    my $self = {};
    bless($self, $class);
    return $self;
}

#-----------------------------------------------------------------------
# Parse SGE Log file
#-----------------------------------------------------------------------
sub parse {
    my $file = shift;
    my $urlist = BATCH2UR::UsageRecordList->new();

    # There are 45 SGE accounting fields as defined in %sgeFields
    # One job per line, fields separated by colons, with position of
    # field denoting its value

    # my %sgeFields=(
#                    "0"=>"qname",
#                    "1"=>"hostname",
#                    "2"=>"group",
#                    "3"=>"owner",
#                    "4"=>"job_name",
#                    "5"=>"job_number",
#                    "6"=>"account",
#                    "7"=>"priority",
#                    "8"=>"submission_time",
#                    "9"=>"start_time",
#                    "10"=>"end_time",
#                    "11"=>"failed",
#                    "12"=>"exit_status",
#                    "13"=>"ru_wallclock",
#                    "14"=>"ru_utime",
#                    "15"=>"ru_stime",
#                    "16"=>"ru_maxrss",
#                    "17"=>"ru_ixrss",
#                    "18"=>"ru_ismrss",
#                    "19"=>"ru_idrss",
#                    "20"=>"ru_isrss",
#                    "21"=>"ru_minflt",
#                    "22"=>"ru_majflt",
#                    "23"=>"ru_nswap",
#                    "24"=>"ru_inblock",
#                    "25"=>"ru_oublock",
#                    "26"=>"ru_msgsnd",
#                    "27"=>"ru_msgrcv",
#                    "28"=>"ru_nsignals",
#                    "29"=>"ru_nvcsw",
#                    "30"=>"ru_nivcsw",
#                    "31"=>"project",
#                    "32"=>"department",
#                    "33"=>"granted_pe",
#                    "34"=>"slots",
#                    "35"=>"task_number",
#                    "36"=>"cpu",
#                    "37"=>"mem",
#                    "38"=>"io",
#                    "39"=>"category",
#                    "40"=>"iow",
#                    "41"=>"pe_taskid",
#                    "42"=>"maxvmem",
#                    "43"=>"arid",
#                    "44"=>"ar_submission_time",
#                );

    open(SGE_LOG, "<$file") or die ("Unable to open $file error: $!");
    while ( my $line = <SGE_LOG>) {
        chomp $line;
        if ( $line =~ m/^#/ ) {
            next;
        };
        my @sge = split(/:/,$line,45);

        #Map to ur from array
        my $ur = $urlist->addRecord($sge[5]);

        $ur->LocalUserId($sge[3]);

        $ur->JobName($sge[4]);
        
        if ($sge[12] eq "0" ) {
            $ur->Status("completed");
        }else {
            $ur->Status("failed");
        }

        $ur->timeSgeSubmitted(epochToDate($sge[8]));

        $ur->WallDuration(secondsToDuration($sge[13]));

        $ur->CpuDuration(secondsToDuration($sge[36]));

        #This function needs to edited on a per-site per-pe
        #basis. For example
        #
        #if (sge[33] = "mpi_pe.4") {
        #    my $procs = $sge[34] * 4;
        #}
        #$ur->Processors($procs);
        #
        #However as a default we assume slots equals processors

        $ur->Processors($sge[34]);

        $ur->StartTime(epochToDate($sge[9]));

        $ur->EndTime(epochToDate($sge[10]));

        $ur->Queue($sge[0]);
        my $vmem=sprintf("%.0f",$sge[42]/1024);
        $ur->virtualMemoryUsed($vmem, "KB");

    };
    return $urlist;
};

1;

__END__

=head1 NAME

SGEParser - Perl extension for parsing the execution records of a
SGE accounting log.

=head1 SYNOPSIS

  use SGEParser;

=head1 ABSTRACT

  SGEParser returns a hash of hashs, keyed by SGE JobID. The data
  hash contains the variables extracted from Execution records from
  the passed in file. These are used by BATCH2UR::Generate to populate the 
  TT template.

=head1 DESCRIPTION

parse() takes one argument the SGE accounting log file.

=head2 EXPORT

None by default.

=head1 SEE ALSO


=head1 AUTHOR

Matt Ford, E<lt>matt.ford@manchester.ac.ukE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2010 by Matt Ford

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

