package BATCH2UR::TorqueParser;

use strict;
use warnings;
use BATCH2UR::ConversionFunctions;
use BATCH2UR::UsageRecordList;

#-----------------------------------------------------------------------
# Constructor (not yet used!!!)
#-----------------------------------------------------------------------
sub new {
    my $class = shift;
    my $self = {};
    bless($self, $class);
    return $self;
}

#-----------------------------------------------------------------------
# Parse Torque Log file
#-----------------------------------------------------------------------
sub parse {
    my $file = shift;
    my $urlist = BATCH2UR::UsageRecordList->new();

    open (TORQUE_LOG, "<$file") or die ("Unable to open $file error: $!");
    while ( my $line = <TORQUE_LOG>) {
	chomp $line;
	(my $logDate, my $type, my $jobID, my $messageArea) = split /;/, $line;
	if ($type eq "E") {

	    my $ur = $urlist->addRecord($jobID);

	    $ur->logDate(pbsLocalToDate($logDate));
	    my @fields=split / /,$messageArea;
	    for my $field (@fields) {
		(my $fieldName, my $fieldValue) = split /=/,$field,2;
		if ( $fieldName eq "user" ) {
		    $ur->LocalUserId($fieldValue);
		};
		if ( $fieldName eq "jobname" ) {
		    $ur->JobName($fieldValue);
		};
		if ( $fieldName eq "Exit_status" ) {
		    if ($fieldValue eq "0") {
			$fieldValue="completed";
		    }else{
			$fieldValue="failed";
		    };
		    $ur->Status($fieldValue);
		    next;
		};
		if ( $fieldName eq "resources_used.mem" ) {
		    ( my $mem = $fieldValue ) =~ s/[a-zA-Z]*$//;
		    ( my $unit = $fieldValue ) =~ s/^[0-9]*//;
		    $unit =~ tr [a-z] [A-Z];
		    $ur->memoryUsed($mem,$unit);
		    next;
		};
		if ( $fieldName eq "resources_used.vmem") {
		    ( my $mem = $fieldValue ) =~ s/[a-zA-Z]*$//;
		    ( my $unit = $fieldValue ) =~ s/^[0-9]*//;
		    $unit =~ tr [a-z] [A-Z];
		    $ur->virtualMemoryUsed($mem,$unit);
		    next;
		};
		if ( $fieldName eq "Resource_List.walltime") {
		    $ur->wallTimeRequested(timeToDuration($fieldValue));
		    next;
		};
		if ( $fieldName eq "Resource_List.cput") {
		    $ur->cpuTimeRequested(timeToDuration($fieldValue));
		    next;
		};
		if ( $fieldName eq "ctime" ) {
		    $ur->timeTorqueSubmitted(epochToDate($fieldValue));
		    next;
		};
		if ( $fieldName eq "qtime" ) {
		    $ur->timeTorqueQueued(epochToDate($fieldValue));
		    next;
		};
		if ( $fieldName eq "etime") {
		    $ur->timeTorqueExecutable(epochToDate($fieldValue));
		    next;
		};
		if ( $fieldName eq "resources_used.walltime" ) {
		    $ur->WallDuration(timeToDuration($fieldValue));
		    next;
		};
		if ( $fieldName eq "resources_used.cput" ) {
		    $ur->CpuDuration(timeToDuration($fieldValue));
		    next;
		};
                # This field is not present in Torque logs
                # Use node count instead....
                #if ( $fieldName eq "resources_used.ncpus" ) {
		#    $ur->Processors($fieldValue);
		#    next;
		#};
		if ( $fieldName eq "end" ) {
		    $ur->EndTime(epochToDate($fieldValue));
		    next;
		};
		if ( $fieldName eq "start" ) {
		    $ur->StartTime(epochToDate($fieldValue));
		    next;
		};
		if ( $fieldName eq "queue" ) {
		    $ur->Queue($fieldValue);
		    next;
		};
		if ( $fieldName eq "exec_host" ) {
		    my @exec_host=split /\+/,$fieldValue;
		    # Torque uses this format
		    #   exec_host=node11/0+node11/1+node12/0
		    # Whereas newer PBS Pro uses this format
		    #   exec_host=node11:ncpus=2+node12:ncpus=1
                    # NodeCount equal to the number of processors...
		    my @nodes= map { my ($x, $r) = split /\//,$_,2 ; $x } @exec_host;
		    $ur->NodeCount(scalar(@nodes));
                    $ur->Processors(scalar(@nodes));
		    $ur->Host(@nodes);
		    next;
		};
	    };
	};
    };
    return $urlist;
};

1;

__END__

=head1 NAME

TorqueParser - Perl extension for parsing the execution records of
Torque accounting log.

=head1 SYNOPSIS

  use TorqueParser;

=head1 ABSTRACT

  TorquParser returns a hash of hashs, keyed by Torque JobID. The data
  hash contains the variables extracted from Execution records from
  the passed in file. These are used by BATCH2UR::Generate to populate the 
  TT template.

=head1 DESCRIPTION

parse() takes one argument the PBS accounting log file.

=head2 EXPORT

None by default.

=head1 SEE ALSO

=head1 AUTHOR

Matt Ford, E<lt>matt.ford@manchester.ac.ukE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2009 by Matt Ford

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut
