package BATCH2UR::UsageRecordList;

use strict;
use BATCH2UR::UsageRecord;

sub new {
    my $class = shift;
    my $self = {};
    bless($self,$class);
    return $self;
}

sub addRecord {
    my $self = shift;
    my $record = shift;    
    if (ref($record) eq 'BATCH2UR::UsageRecord') {
	$self->{$record}->{JobId} = $record;
    }
    else {
	$self->{$record} = BATCH2UR::UsageRecord->new();
	$self->{$record}->{JobId} = $record;
    }
    return $self->{$record};
}

sub deleteRecord {
    my $self = shift;
    my $jobid = shift;
    my $record = $self->{$jobid};
    delete( $self->{$jobid} );
    return $record;
}

sub getRecord {
    my $self = shift;
    my $jobid = shift;
    return $self->{$jobid};
}

sub jobIDList {
    my $self = shift;
    my @jobIDList = keys( %{ $self } );
    return @jobIDList;
}

sub length { 
    my $self = shift;
    return scalar( keys( %{$self} ));
}

1;

__END__

=head1 NAME

BATCH2UR::UsageRecordList - Class for storing Job Scheduler Usage Records

=head1 SYNOPSIS

  use BATCH2UR::UsageRecordList;

  my ($rl, $ur);

  # Create a new RecordList
  $rl = BATCH2UR::UsageRecordList->new();

  # Create a new UsageRecord
  $ur = $rl->addRecord('x3456');

  # Add Data to UsageRecord
  $ur->localUserID('ngs005');
  $ur->status('complete');
  $ur->nodeCount(28);
  $ur->toString();

  # Create a second UsageRecord
  $ur = $rl->addRecord('xxx1');

  # Print number of Records in List
  print $rl->length();

=head1 ABSTRACT

BATCH2UR::UsageRecordList is a container class for UsageRecord objects, 
you can instantiate UsageRecord objects separately and then add them
to a UsageRecordList, or you can instantiate them on-the-fly.

=head1 METHODS

=head2 new

Creates a new UsageRecordList object.

=head2 length

Returns an integer equal to the number of records in the 
UsageRecordList.

=head2 addRecord(JobID)

Instantiates a new UsageRecord object using the given JobID, and then 
adds it to the UsageRecordList.  Returns the new UsageRecord as a side 
effect

=head2 addRecord(UsageRecord)

Adds an existing UsageRecord object to the UsageRecordList. Returns the 
UsageRecord as a side effect.

=head2 deleteRecord(jobID)

Deletes the UsageRecord, with the given JobID, from the UsageRecordList.
This method also returns a copy of the deleted UsageRecord.

=head2 getRecord(jobID)

Returns the UsageRecord, with the given JobID, from the UsageRecordList.

=head2 jobIDList

Returns an array containing the JobIDs of all of the Records in the 
UsageRecordList.

=head2 EXPORT


=head1 SEE ALSO

BATCH2UR::UsageRecord - a module to create the individual records that are
stored in a UsageRecordList

=head1 AUTHOR

Jon Wakelin, E<lt>isxjw@bris.ac.ukkE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2006 by Jon Wakelin

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself. 

=cut
